/*
 * Copyright (C) 2021  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "src/grpc_client/get_acceptable_keysets.hh"
#include "src/grpc_client/make_stub.hh"
#include "src/grpc_client/proto_unwrap.hh"
#include "src/exception.hh"

Fred::Akm::AcceptableKeysets Fred::Akm::GrpcClient::get_acceptable_keysets(
        const SupervisorConnectionString& _supervisor_connection_string)
{
    namespace Api = CdnskeyProcessor::Api::Supervisor;
    using ::google::protobuf::Empty;

    auto stub = make_stub(_supervisor_connection_string);
    grpc::ClientContext context;

    AcceptableKeysets acceptable_keysets;
    Api::AcceptableDnskeysReply acceptable_dnskeys_api;
    auto reader = stub->acceptable_dnskeys(&context, Empty{});
    while (reader->Read(&acceptable_dnskeys_api))
    {
        merge(acceptable_keysets, proto_unwrap(acceptable_dnskeys_api));
    }

    const auto status = reader->Finish();

    if (!status.ok())
    {
        throw GrpcException{status.error_message()};
    }

    return acceptable_keysets;
}
