/*
 * Copyright (C) 2021-2024  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "src/scan_type.hh"

#include <stdexcept>

namespace Fred {
namespace Akm {

template<>
ScanType from_string(const std::string& _str)
{
    if (to_string(ScanType::insecure) == _str)
    {
        return ScanType::insecure;
    }
    if (to_string(ScanType::secure) == _str)
    {
        return ScanType::secure;
    }
    if (to_string(ScanType::secure_auto) == _str)
    {
        return ScanType::secure_auto;
    }
    throw std::invalid_argument("cannot convert \"" + _str + "\" to ScanType");
}

} // namespace Fred::Akm
} // namespace Fred

using namespace Fred::Akm;

std::string Fred::Akm::to_string(ScanType _scan_type)
{
    switch (_scan_type)
    {
        case ScanType::insecure:
            return "insecure";
        case ScanType::secure:
            return "secure";
        case ScanType::secure_auto:
            return "secure-auto";
    }
    throw std::logic_error{"unexpected ScanType"};
}

std::string Fred::Akm::to_string(ExportScanType scan_type)
{
    switch (scan_type)
    {
        case ExportScanType::all:
            return "all";
        case ExportScanType::insecure:
            return "insecure";
        case ExportScanType::secure:
            return "secure";
        case ExportScanType::secure_auto:
            return "secure-auto";
    }
    throw std::logic_error{"unexpected ExportScanType"};
}
